from django.db import models
from jungle.organization.models import Organization

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

##############################
# CONTRATTO                  #
##############################
class Contract (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        verbose_name="Ragione Sociale",
    )
    periodicity = models.ForeignKey (
        "ContractPeriodicity",
        verbose_name="Periodicita' Contratto",
    )
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    title = models.CharField (
        max_length=100,
        verbose_name="Titolo",
    )
    description = models.TextField (
        blank=True,
        default="",
        verbose_name="Descrizione",
    )
    date_agreement = models.DateTimeField (
        verbose_name="Data di Stipula",
    )
    days_before = models.IntegerField (
        verbose_name="Giorni Recesso",
    )
    note = models.TextField (
        blank=True,
        default="",
        verbose_name="Note",
    )
    class Admin:
        list_filter = ("status", "periodicity",)
        search_fields = ("number", "date_agreement", )
        date_hierarchy = "date_create"
        list_display = ("status", "organization", "periodicity", "code", "title", "date_agreement", "days_before",)
        list_display_links = ("code", "title",)
        list_per_page = 30
        ordering = ("-code",)
        fields = (
            (None, {"fields": (
                ("status",),
                ("organization",),
                ("periodicity",),
                ("code", "title",),
                ("description",),
                ("date_agreement",),
                ("days_before",),
                ("note",),
            ),}),
        )

    class Meta:
        verbose_name = "Contratto"
        verbose_name_plural = "Contratti"
    
    def __unicode__(self):
        return unicode(self.title+" "+self.organization.name)    

#################################
# RIGA CONTRATTO                #
#################################
class ContractRow (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    contract = models.ForeignKey (
        Contract,
        edit_inline=models.TABULAR,
        min_num_in_admin=1,
        num_extra_on_change=1,
        num_in_admin=1,
        verbose_name="Contratto",
    )
    type = models.ForeignKey (
        "ContractType",
        verbose_name="Tipo Contratto",
    )
    ## Campi
    date_begin = models.DateField (
        verbose_name="Data Inizio",
        core=True,
    )
    date_end = models.DateField (
        verbose_name="Data Fine",
    )
    description = models.TextField (
        blank=True,
        default="",
        verbose_name="Descrizione",
    )
    amount = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Importo",
        default=0.00,
    )

    class Admin:
        list_filter = ("status", "type", "contract",)
        search_fields = ("date_begin", "date_end", "description",)
        date_hierarchy = "date_create"
        list_display = ("status", "date_begin", "date_end", "contract", "type", "amount",)
        list_display_links = ("type",)
        list_per_page = 30
        ordering = ("date_begin", "contract", "type",)
        fields = (
            (None, {"fields": (
                ("status",),
                ("date_begin", "date_end",),
                ("contract", "type", "amount",),
                ("description",),
            ),}),
        )

    class Meta:
        verbose_name = "Riga Contratto"
        verbose_name_plural = "Righe Contratto"

    def __unicode__(self):
        return unicode(self.description)    

##################################
# TIPO CONTRATTO                 #
##################################
class ContractType (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        max_length=100,
        verbose_name="Tipo Contratto",
    )
    note = models.TextField (
        blank=True,
        default="",
        verbose_name="Note",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name",)
        list_display_links = ("code", "name",)
        list_per_page = 30
        ordering = ("odv","name",)
        fields = (
            (None, {"fields": (
                ("status", "code", "name", "odv",),
                ("note",),
            ),}),
        )

    class Meta:
        verbose_name = "Tipo Contratto"
        verbose_name_plural = "Tipi Contratto"
    
    def __unicode__(self):
        return unicode(self.name)    

#########################################
# PERIODICITA' CONTRATTO                #
#########################################
class ContractPeriodicity (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        max_length=100,
        verbose_name="Tipo Periodicita'",
    )
    ss = models.SmallIntegerField (
        max_length=2,
        default=0,
        verbose_name="N. Settimane",
    )
    mm = models.SmallIntegerField (
        max_length=2,
        default=0,
        verbose_name="N. Mesi",
    )
    aa = models.SmallIntegerField (
        max_length=2,
        default=0,
        verbose_name="N. Anni",
    )
    note = models.TextField (
        blank=True,
        default="",
        verbose_name="Note",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "ss", "mm", "aa",)
        search_fields = ("code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name","ss", "mm", "aa",)
        list_display_links = ("code", "name",)
        list_per_page = 30
        ordering = ("odv","name",)
        fields = (
            (None, {"fields": (
                ("status", "code", "name", "odv",),
                ("ss", "mm", "aa",),
                ("note",),
            ),}),
        )

    class Meta:
        verbose_name = "Periodicita' Contratto"
        verbose_name_plural = "Periodicita' Contratto"

    def __unicode__(self):
        return unicode(self.name)    