from django.newforms.models import form_for_model, form_for_instance
from django.newforms import Form
from jungle.mailserver.models import Account, Domain, Alias, Vacation
from jungle.db.formfield import ffc
from django.newforms.fields import *
import re
from jungle.middleware.thread_local import get_current_user
from django.contrib.auth.models import Group
from django.newforms.util import ValidationError

from django import newforms

#from jungle.functions import log
from django.db import transaction
from django.core.validators import ip4_re

TRANSPORT_VALUE = ('virtual', 'vacation:',)

class DomainForm(form_for_model(Domain, formfield_callback=ffc())):


    def clean_quota(self):
        quota = self.cleaned_data['quota'] 
        if quota and quota < 0:
            raise ValidationError(u"Quota can be blank or integer >= 0")
        return quota
        
    def clean_transport(self):
        transport = self.cleaned_data['transport']
        # default for blank
        if not transport:
            transport = 'virtual'

        # transport is an IP or a TRANSPORT_VALUE
        elif ip4_re.search(transport):
            transport = 'smtp:[%s]' % transport
            
        elif transport in TRANSPORT_VALUE:
            pass
            
        elif re.match('smtp:\[.*\]', self.cleaned_data['transport']):
            pass
        else:
            raise ValidationError(
                u"Transport has to be in the form of an " +
                "IP Address. Blank for the standard 'virtual'.")

        return transport
        
    def clean(self):

        domains = Domain.objects.filter(
            name__exact=self.cleaned_data["name"]).count()
        if domains > 0 and 'id' not in self.data:
            raise ValidationError(u"Existing domain.")

        return self.cleaned_data

    def save(self):
        if 'id' in self.data:
            ## update 
            domain = Domain.objects.get(pk=self.data.get('id'))
            f = form_for_instance(domain, formfield_callback=ffc())(self.data)
            f.save()
        else:
            # save
            domain = super(DomainForm, self).save()

        return domain

class AccountForm(form_for_model(Account, formfield_callback=ffc())):

    def __init__(self, *args, **kwargs):
        super(AccountForm, self).__init__(*args, **kwargs)
        self.fields['domain'].queryset = Domain.objects.all()

    def clean_quota(self):
        quota = self.cleaned_data['quota'] 
        if quota and quota < 0:
            raise ValidationError(u"Quota can be blank or integer >= 0")
        return quota
        
    def clean(self):

        if 'account' in self.cleaned_data:
            accounts = Account.objects.filter(
                account__exact=self.cleaned_data["account"],
                domain=self.cleaned_data["domain"]).count()

        if 'domain' in self.cleaned_data:
            if accounts > 0:
                if 'id' not in self.data:
                    raise ValidationError(u"Existing account.")

            accounts=Account.objects.filter(domain=self.cleaned_data["domain"])
            num_accounts = accounts.count()

            used = 0
            for a in accounts:
                if a.transport == 'virtual':
                    if 'id' not in self.data:
                        used += a.quota or 0
                    elif a.id != self.data["id"]:
                        used += a.quota or 0
                    else:
                        num_accounts-=1

            domain = self.cleaned_data['domain']

            try:
                domain_quota = int(domain.quota)
            except:
                domain_quota = 0

            try:
                account_quota = int(self.cleaned_data["quota"])
            except:
                self.cleaned_data["quota"] = 0
                account_quota = 0

            if account_quota == 0 and domain_quota > 0:
                raise ValidationError(u"Can't set unlimited account quota.")

            elif account_quota > 0 and domain_quota > 0:
                if (domain_quota - used) < account_quota:
                    raise ValidationError(
                        u"Domain quota is not enough to create a domain.")

            try:
                limit = int(self.cleaned_data['domain'].users)
            except:
                limit = num_accounts + 1

            if (limit - num_accounts) < 1:
                raise ValidationError(
                    u"Domain accounts limit is not enough to create a domain.")

        return self.cleaned_data

    @transaction.autocommit
    def save(self):
        if 'id' in self.data:
            ## update
            account = Account.objects.get(pk=self.data.get('id'))
            f = form_for_instance(account, formfield_callback=ffc())(self.data)
            f.save()
        else:
            # save
            account = super(AccountForm, self).save()

            from django.core.mail import send_mail
            send_mail('wellcome', '\n\nWellcome/Benvenuto\n\nAccount attivato', 'noreply@mailserver.com',
                      ['%s@%s' % (account.account, account.domain.name)])
        
        return account

class AliasForm(form_for_model(Alias, formfield_callback=ffc())):

    def clean_alias(self):
        alias = self.cleaned_data['alias']
        if '@' in alias:
            raise ValidationError(u"Alias is not an email address. Check it.")

        return alias

    def save(self):
        #form used just for update
        #if 'id' in self.data:
        alias = Alias.objects.get(pk=self.data.get('id'))
        f = form_for_instance(alias, formfield_callback=ffc())(self.data)
        f.save()

        return alias

def get_domain_field():

    return AliasForm()['domain'].field

class AliasesForm(form_for_model(Alias, formfield_callback=ffc())): # to add aliases


    def __init__(self, *args, **kwargs):

        super(AliasesForm, self).__init__(*args, **kwargs)
        self.fields['domain'].queryset = Domain.objects.all()
        self.fields['alias'] = CharField(max_length = 127)
        self.fields['addresses'] = CharField({'required' : False})
        self.fields['address'].required = False

    def clean(self):
        if 'alias' in self.cleaned_data and '@' in self.cleaned_data['alias']:
            raise ValidationError(u"Alias is not an email address. Check it.")

        if 'addresses' in self.cleaned_data and 'domain' in self.cleaned_data:
            addresses = re.split('\r|\n|\t|:|;|\,', self.cleaned_data['addresses'].replace(' ', '\n'))

            num_aliases_to_create = len(addresses)

            num_existing_aliases = Alias.objects.filter(domain = self.cleaned_data['domain']).count()

            try:
                limit = int(self.domain.aliases)
            except:
                limit = num_existing_aliases + num_aliases_to_create

            if (limit - num_existing_aliases) < num_aliases_to_create:
                raise ValidationError(u"Domain aliases limit is not enough to create the following aliases.")

        return self.cleaned_data

    def save(self):
        addresses = re.split('\r|\n|\t|:|;|\,', self.cleaned_data['addresses'].replace(' ', '\n'))

        for add in addresses:
            if add:
                a = Alias()
                a.domain  = self.cleaned_data['domain']
                a.alias    = self.cleaned_data['alias']
                a.address = add
                a.status = 1 # active
                a.save()

        return a # returning the last created


class VacationForm(form_for_model(Vacation, formfield_callback=ffc())):
    def __init__(self, *args, **kwargs):
        super(VacationForm, self).__init__(*args, **kwargs)
        self.fields['account'] = newforms.ModelChoiceField(queryset=Account.objects.all())

