from django.db import models
from jungle.userprofile.models import Office
from jungle.address.models import Address, AddressScope, AddressType

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

ORGANIZATION_TYPE = (
    ("Azienda", "Azienda",),
    ("Cliente","Cliente",),
    ("Fornitore","Fornitore",),
)

################################
# ANAGRAFICA                   #
################################
class Organization (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    office = models.ForeignKey (
        Office,
        null=True, blank=True,
        verbose_name="Ufficio",
    )
    address = models.ManyToManyField(
        Address,
        null=True, blank=True,
        verbose_name="Indirizzo",
        raw_id_admin=True,
    )
    organization_group = models.ManyToManyField(
        "OrganizationGroup",
        verbose_name="Gruppo",
        related_name="group",
        null=True,
        blank=True,
    )
    organization_type = models.ForeignKey (
        "OrganizationType",
        verbose_name="Tipo Anagrafica",
    )
    organization_form = models.ForeignKey (
        "OrganizationForm",
        verbose_name="Forma Societaria",
    )
    field = models.ForeignKey (
       "Field",
        verbose_name="Settore",
    )
    #list_type = models.ForeignKey (
       #ListType1,
        #verbose_name="Tipo Listino",
    #)
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice Anagrafica",
    )
    name = models.CharField (
        max_length=100,
        verbose_name="Ragione Sociale",
    )
    brand = models.CharField (
        max_length=255,
        null=True,
        blank=True,
        verbose_name="Insegna",
    )
    vat_number = models.CharField (
        max_length=11,
        null=True,
        blank=True,
        verbose_name="Partita IVA",
    )
    code_tax = models.CharField (
        max_length=16,
        null=True,
        blank=True,
        verbose_name="Codice Fiscale",
    )
    url = models.URLField (
        max_length=100,
        null=True,
        blank=True,
        verify_exists=True,
        verbose_name="URL",
    )
    email = models.EmailField (
        max_length=100,
        null=True,
        blank=True,
        verbose_name="Email",
    )
    note = models.TextField (
        null=True,
        blank=True,
        verbose_name="Note",
    )
    phone = models.CharField (
        max_length=20,
        verbose_name="Telefono",
    )
    phone2 = models.CharField (
        null=True,
        blank=True,
        max_length=20,
        verbose_name="Telefono",
    )
    fax = models.CharField (
        null=True,
        blank=True,
        max_length=20,
        verbose_name="Fax",
    )

    class Admin:
        list_filter = ("status", "office", "organization_type", "organization_form", "field",)
        search_fields = ("^organization_type", "^name", "^phone", )
        date_hierarchy = "date_create"
        list_display = ("status", "office", "brand", "organization_type", "field", "code", "name", "url", "email", "phone", "fax", )
        list_display_links = ("code", "name", )
        list_per_page = 30
        ordering = ("name","-code",)
        fields = (
            ("*** CODICI ***", {"fields": (
                ("status", "code", ),
            ),}),
            ("*** ANAGRAFICA ***", {"fields": (
                ("office","organization_type",),
                ("name", "organization_form","field",),
                ("brand",),
                ("vat_number", "code_tax",),
                ("organization_group",),
            ),}),
            ("*** CONTATTI ***", {"fields": (
                ("url", "email"),
                ("phone", "phone2", "fax"),
            ),}),
            ("*** INDIRIZZO ***", {"fields": (
                ("address",)
            ),}),
            ("*** NOTE ***", {"fields": ("note",)}),
        )

    class Meta:
        unique_together = (
            ("vat_number", "code_tax",),
        )
        verbose_name = "Anagrafica"
        verbose_name_plural = "Anagrafiche"

    def __unicode__(self):
        return unicode(self.name)
    
##################################
# GRUPPO SOCIETA                 #
##################################
class OrganizationGroup (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    organization = models.ManyToManyField(
        "Organization",
        verbose_name="Anagrafica",
        #db_table="organization_organization_organization_group",
        null=True,
        blank=True,
    )
  ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Gruppo Societa'",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )
    class Admin:
        pass

    class Meta:
        verbose_name = "Gruppo Societa'"
        verbose_name_plural = "Gruppi Societa'"

    def __unicode__(self):
        return unicode(self.name)
    
##################################
# FORMA SOCIETA                  #
##################################
class OrganizationForm (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Forma Societaria",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name",)
        list_per_page = 20
        fields = (
            (None, {"fields": (
                ("status", "name", "odv",),
            ),}),
        )

        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Forma Societaria"
        verbose_name_plural = "Forme Societarie"

    def __unicode__(self):
        return unicode(self.name)
    
####################################
# TIPO ANAGRAFICA                  #
####################################
class OrganizationType (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    type = models.CharField (
        max_length=15,
        choices=ORGANIZATION_TYPE,
        verbose_name="Categoria",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Nome",
    )
    description = models.TextField (        
        null=True,
        blank=True,
        default="",
        verbose_name="Descrizione",
    )
    note = models.TextField (
        null=True,
        blank=True,
        default="",
        verbose_name="Note",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name", 'type',)
        date_hierarchy = "date_create"
        list_display = ("status", "code","type","name",)
        list_display_links = ("name",)
        list_per_page = 20
        fields = (
            (None, {"fields": (
                ("status", "code", "type", "name",),
                ("description",),
                ("note",),
                ("odv",),
            ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Tipo Anagrafica"
        verbose_name_plural = "Tipo Anagrafiche"

    def __unicode__(self):
        return unicode(self.code+" "+self.type+" "+self.name)
    
#############################
# SETTORE                   #
#############################
class Field (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Settore",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name",)
        list_per_page = 20
        fields = (
            (None, {"fields": (
                ("status", "name", "odv",),
            ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Settore"
        verbose_name_plural = "Settori"

    def __unicode__(self):
        return unicode(self.name)
    
###################################
# CONTATTO RUOLO                  #
###################################
class TitlePerson (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Ruolo",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                ("status", "name", "odv",),
            ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Ruolo Contatto"
        verbose_name_plural = "Ruoli Contatto"

    def __unicode__(self):
        return unicode(self.name)
    
##############################
# CONTATTO                   #
##############################
class Person (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    organization = models.ManyToManyField (
        Organization,
        null=True, blank=True,
        verbose_name="Ragione Sociale",
    )
    title = models.ForeignKey (
        TitlePerson,
        verbose_name="Ruolo",
        num_in_admin=3,
        core=True,
    )
    ## Campi
    lastname = models.CharField (
        max_length=100,
        verbose_name="Cognome",
        core=True,
    )
    firstname = models.CharField (
        max_length=100,
        verbose_name="Nome",
        core=True,
    )
    phone = models.CharField (
        max_length=20,
        verbose_name="Telefono",
        core=True,
    )
    mobile = models.CharField (
        max_length=20,
        verbose_name="Cellulare",
        core=True,
    )
    fax = models.CharField (
        max_length=20,
        verbose_name="Fax",
    )
    email = models.EmailField (
        max_length=100,
        null=True,
        blank=True,
        verbose_name="Email",
        core=True,
    )
    note = models.TextField (
        max_length=255,
        null=True,
        blank=True,
        verbose_name="Note",
    )
    ## fotoContatto

    class Admin:
        list_filter = ("status", "title",)
        search_fields = ("organization", "lastname", "title", "phone", "mobile",)
        date_hierarchy = "date_create"
        list_display = ("status", "lastname", "firstname", "title", "phone", "fax", "mobile", "email",)
        list_display_links = ("lastname",)
        list_per_page = 20
        fields = (
            (None, {"fields": (
                ("status", "title", "organization",),
                ("lastname", "firstname",),
                ("phone", "mobile", "fax", "email",),
                ("note",)
            ),}),
        )

        ordering = ("lastname","Title",)

    class Meta:
        verbose_name = "Contatto"
        verbose_name_plural = "Contatti"

    def __unicode__(self):
        return unicode(self.lastname+" "+self.firstname)
    
##############################
# BANCA                      #
##############################
class Bank(models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    code_ABI = models.CharField (
        max_length=5,
        verbose_name="ABI",
    )
    code_CAB = models.CharField (
        max_length=5,
        verbose_name="CAB",
    )
    ## code_ABI + code_CAB sono una ForeignKey a ref->refBankCAB
    ## Campi
    code_CIN = models.CharField (
        max_length=1,
    )
    bank_account = models.CharField (
        max_length=12,
    )
    organization = models.ManyToManyField (
        Organization,
        verbose_name="Ragione Sociale",
    )

    class Admin:
        pass

    class Meta:
        verbose_name = "Banca"
        verbose_name_plural = "Banche"
        unique_together = (
            ("code_ABI", "code_CAB"),
        )

    def __unicode__(self):
        return unicode(self.bank_account)
