from django.db import models
from django.contrib.auth.models import Group, User

# Create your models here

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)
class MenuManager(models.Manager):
    def get_query_set(self):
        from jungle.middleware.thread_local import get_current_user
        self.user = get_current_user()
        if self.user and self.user.is_authenticated() and self.user.is_staff:
            return super(MenuManager, self).get_query_set()
        else :
            return super(MenuManager, self).get_query_set().filter(status='1')
        


############################
# MENU                     #
############################
class Menu (models.Model):
    objects = MenuManager()
    all_objects = models.Manager()
    
    name = models.CharField (
        max_length=50,
        verbose_name="Nodo",
    )
    description = models.CharField (
        max_length=50,
        verbose_name="Descrizione",
    )
    path = models.CharField (
        max_length=200,
        verbose_name="Path",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    father = models.ForeignKey (
        "self",
        verbose_name="Nodo Padre",
        blank=True,
        null=True,
    )
    level = models.IntegerField (
        verbose_name="Livello",
    )
    extra = models.CharField (
        blank=True,
        max_length=50,
        verbose_name="Extra",
    )

    class Admin:
        list_filter = ('status', 'father', )
        search_fields = ('name', 'path',)
        list_per_page = 30
        list_display = ('status', 'name', 'father', 'path',)
        list_display_links = ('name',)

    class Meta:
        unique_together = (("name", "path", "father", "level"),)
        verbose_name = "Menu"
        verbose_name_plural = "Menu"

    def __unicode__(self):
        return unicode(self.name)
    
#############################
# STATO                     #
#############################
class Status (models.Model):
    name = models.CharField (
        unique=True,
        max_length=50,
        verbose_name="Stato",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )
    obs = models.BooleanField (
        default=0,
        verbose_name="Obsoleto"
    )

    class Admin:
        list_filter = ('obs',)
        search_fields = ('name',)
        list_per_page = 30
        list_display = ('name', 'obs', 'odv',)
        list_display_links = ('name',)
        ordering = ('odv', 'obs', 'name',)

    class Meta:
        verbose_name = "Stato"
        verbose_name_plural = "Stati"

    def __unicode__(self):
        return unicode(self.name)
    
###########################
# GRUPPO                  #
###########################
class GroupExt (models.Model):
    dgroup = models.ForeignKey (
        Group,
        verbose_name="Gruppo",
    )
    level = models.IntegerField (
        default=1,
        verbose_name="Livello",
    )
    father = models.ForeignKey (
        "self",
        verbose_name="Gruppo Padre",
        blank=True,
        null=True,
    )

    class Admin:
        list_filter = ('dgroup', 'level',)
        search_fields = ('dgroup', 'father', 'level',)
        list_per_page = 30
        list_display = ('dgroup', 'father', 'level',)
        list_display_links = ('dgroup',)
        ordering = ('level', 'dgroup',)

    class Meta:
        unique_together = (
            ('dgroup', 'level', 'father',),
        )
        verbose_name = "Gruppo"
        verbose_name_plural ="Gruppi"

    def __unicode__(self):
        return unicode(self.dgroup.name)

#########################
# PERM                  #
#########################
class Perm (models.Model):
    table_name = models.CharField (
        max_length=50,
        verbose_name="Tabella",
    )
    field_name = models.CharField (
        max_length=50,
        verbose_name="Campo",
    )
    dgroup = models.ForeignKey (
        Group,
        verbose_name="Gruppo",
    )
    status = models.ForeignKey (
        Status,
        verbose_name="Stato",
    )
    # 0001->1: SELECT - 0010->2: INSERT - 0100->4: UPDATE - 1000->8: DELETE
    perms = models.IntegerField (
        default='0',
        max_length=2,
    )

    class Admin:
        list_filter = ('table_name', 'dgroup', 'status',)
        search_fields = ('field_name',)
        list_per_page = 30
        list_display = ('table_name', 'field_name', 'dgroup', 'status', 'perms',)
        list_display_links = ('field_name',)
        ordering = ('table_name', 'field_name', 'dgroup', 'status',)

    class Meta:
        unique_together = (
            ('table_name', 'field_name', 'dgroup', 'status',),
        )
        verbose_name="Permesso"
        verbose_name_plural="Permessi"

#########################
# GERARCHIA             #
#########################
class Hier (models.Model):
    table_name = models.CharField (
        max_length=50,
        verbose_name="Tabella",
    )
    field_name = models.CharField (
        max_length=50,
        verbose_name="Campo",
    )
    father = models.ForeignKey (
        User,
        related_name="user_father",
        verbose_name="Utente Master",
    )
    son =  models.ForeignKey (
        User,
        related_name="user_son",
        verbose_name="Utente Slave",
    )

    class Admin:
        list_filter = ('table_name', 'field_name',)
        search_fields = ('father', 'son')
        list_per_page = 30
        list_display = ('table_name', 'field_name', 'father', 'son',)
        list_display_links = ('field_name',)
        ordering = ('table_name', 'field_name', 'father', 'son',)

    class Meta:
        unique_together = (
            ('table_name', 'field_name', 'father', 'son',),
        )
        verbose_name="Gerarchia"
        verbose_name_plural="Gerarchie"

#########################
# GESTIONE              #
#########################
class Man (models.Model):
    table_name = models.CharField (
        max_length=50,
        verbose_name="Tabella",
    )
    dgroup = models.ForeignKey (
        Group,
        verbose_name="Gruppo",
    )
    status = models.ForeignKey (
        Status,
        verbose_name="Stato",
    )

    class Admin:
        list_filter = ('table_name', 'dgroup', 'status')
        search_fields = ()
        list_per_page = 30
        list_display = ('table_name', 'dgroup', 'status')
        list_display_links = ('table_name',)
        ordering = ('table_name', 'dgroup', 'status')

    class Meta:
        unique_together = (
            ('table_name', 'dgroup', 'status',),
        )
        verbose_name="Gestione"
        verbose_name_plural="Gestioni"

