# coding: latin-1
from django.db import models
from django.contrib.auth.models import User
#from jungle.registration.models import UserProfile
from jungle.organization.models import Organization
#from jungle.file.models         import File
from django.conf import settings
from django.db.models import signals
from django.dispatch import dispatcher
from jungle.ticket.jsignals import evaluate_and_save, mail_on_change
from jungle.templatetags.jungle_tags import ugettext as _
#from django.utils.translation import gettext_lazy as _

STATUS_CODES = (
    (0, _('closed')),
    (1, _('open')),
    (2, _('standby')),
    (3, _('waiting for input')),
    (4, _('not repeatable')),
)

PRIORITY_CODES = (
    (0, _('very urgent')),
    (1, _('urgent')),
    (2, _('important')),
    (3, _('normal')),
    (4, _('low')),
    (5, _('someday')),
)

CATEGORY_CODES = (
    ('fix',      _('bug fix')),
    ('new',      _('new feature')),
    ('test',     _('test')),
    ('doc',      _('documentation')),
    ('decis', _('decision')),
)

class ProjectManager(models.Manager):
    def get_query_set(self):
        from django.db.models import Q
        from jungle.middleware.thread_local import get_current_user

        self.user = get_current_user()
        original = super(ProjectManager, self).get_query_set()
        ## authenticated
        if self.user and self.user.is_authenticated():
            if self.user.is_staff:
                return original
            else:
                Q1 = Q(staff = self.user)
                Q2 = Q(manager = self.user)
                Q3 = Q(ticket__assigned_to = self.user)
                return original.filter(Q1 | Q2 ).distinct()
#                 Q4 = Q(report__user = self.user)
#                return original.filter(Q1 | Q2 | Q3 | Q4).distinct()
        else:
            return original.filter(status=-100)
        
class TicketManager(models.Manager):
    def get_query_set(self):
        from django.db.models import Q
        from jungle.middleware.thread_local import get_current_user

        self.user = get_current_user()
        original = super(TicketManager, self).get_query_set()
        ## authenticated
        if self.user and self.user.is_authenticated():
            if self.user.is_staff:
                return original
            else:
                Q1 = Q(project__staff = self.user)
                Q2 = Q(project__manager = self.user)
                Q3 = Q(assigned_to = self.user)
                return original.filter(Q1 | Q2 | Q3).distinct()
        else:
            return original.filter(status=-100)
        
class Ticket(models.Model):
    """Trouble tickets"""
    objects = TicketManager()
    
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Creation Date",
        null=True,
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Ultima Modifica",
        null=True,
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Status",
    )
    ## ForeignKey
    project = models.ForeignKey (
        'Project',
        verbose_name="Project",
    )
    submitter = models.ForeignKey (
        User,
        related_name="submitter",
        verbose_name="Submitter",
    )
    assigned_by = models.ForeignKey (
        User,
        blank=True,
        null=True,
        related_name="assigned_by",
        verbose_name="Assigned by",
    )
    assigned_to = models.ForeignKey (
        User,
        blank=True,
        null=True,
        verbose_name="Assigned to",
    )
    ## Campi
    title = models.CharField (
        max_length=100,
        verbose_name="Title",
    )
    keywords = models.CharField (
        max_length=150,
        verbose_name="Keywords",
        help_text="E' necessario mettere alcune parole utili " +\
             "per la catalogazione del ticket"
    )
    description = models.TextField (
        verbose_name="Description",
    )
    priority = models.IntegerField (
        default=1,
        choices=PRIORITY_CODES,
        verbose_name="Priority",
    )
    public = models.BooleanField (
        default=False,
        verbose_name="Pubblico",
    )
    duration  = models.IntegerField (
#        max_digits=4,
#        decimal_places=2,
        null=True,
        blank=True,
        verbose_name="Duration",
    )
    category = models.ForeignKey('Category', to_field="code",
                                 db_column='category_code')
    deadline = models.DateField(null=True, blank=True)

    class Admin:
        pass
        list_filter = ("status", "project", "assigned_to", "priority",)
        search_fields = ('title', 'description',)
        date_hierarchy = "date_create"
        list_display = ('title', 'priority', 'status', 'submitter', 'assigned_to', 'date_create', 'date_last_modify', "duration",)
        list_display_links = ("title",)
        list_per_page = 30
        ordering = ("status", "priority", "date_create", "title")
        fields = (
            (None, {"fields": (
                ("status", "priority", "public",),
                ("submitter", "assigned_to",),
                ("project", "title",),
                ("keywords",),
                ("description",),
                ("date_create",),
                ("date_last_modify",),
                ("duration", "priority",),
            ),}),
        )

    class Meta:
        verbose_name = "Ticket"
        verbose_name_plural = "Tickets"

    def __unicode__(self):
        return unicode(self.title)
    
############################
# PROGETTO                 #
############################
PSTATUS_CODES = (
    (1, 'Open'),
    (0, 'Closed'),
)

class Project(models.Model):
    """Project: each user can have multiple projects"""

    ## Managers
    objects     = models.Manager()
    my_objects  = ProjectManager()
    all_objects = models.Manager()

    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Creation Date",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=PSTATUS_CODES,
        verbose_name="Status",
    )
    ## ForeignKey
    organization = models.ManyToManyField (
        Organization,
        verbose_name="Organization",
        filter_interface=models.HORIZONTAL,
    )
    manager = models.ManyToManyField (
        User, related_name='Manager', verbose_name='Responsabile')

    staff = models.ManyToManyField (
        User,
        verbose_name="Staff",
        filter_interface=models.HORIZONTAL,
    )
    ## Campi
    name = models.CharField (
        max_length=30,
        verbose_name="Project",
    )
    description = models.TextField (
        null=True,
        verbose_name="Description",
    )
    date_start = models.DateField (
        verbose_name="Start Date",
    )
    date_end = models.DateField (
        verbose_name="End Date",
        null=True,
        blank=True,
    )

    class Admin:
        list_filter = ("status", "staff",)
        search_fields = ("^organization", "^name", "date_start", )
        date_hierarchy = "date_create"
        list_display = ("name", "date_start", "date_end", )
        list_display_links = ("name", )
        list_per_page = 30
        ordering = ("name","-date_start",)
        fields = (
            (None, {"fields": (
                ("status",),
                ("organization", ),
                ("staff",),
                ("name",),
                ("description",),
                ("date_start", "date_end",)
            ),}),
        )

    class Meta:
        verbose_name = "Progetto"
        verbose_name_plural = "Progetti"
        ordering = ['name']

    def __unicode__(self):
        return unicode(self.name)  

###########################
# AZIONE                  #
###########################
STATUS_CODES = (
    (0, 'Disable'),
    (1, 'Enable'),
)

class Action(models.Model):
    """Each action refers to a ticket (indirectly to a project)"""
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Creation date",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Status",
    )
    ## ForeignKey
    assigned_to = models.ForeignKey (
        User,
        blank=True,
        null=True,
        verbose_name="Assigned to",
    )
    ticket = models.ForeignKey(Ticket)
    ## Campi
    name = models.CharField (
        max_length=30,
        verbose_name="Action",
    )
    description = models.TextField (
        max_length=100,
        null=True,
        blank=True,
        verbose_name="Description",
    )
#     comment = models.TextField (
#         max_length=300,
#         null=True,
#         blank=True,
#         verbose_name="Comment",
#     )
    duration  = models.IntegerField (
#        max_digits=4,
#        decimal_places=2,
        null=True,
        blank=True,
        verbose_name="Duration",
    )
#     duration = models.DecimalField (
#         max_digits=4,
#         decimal_places=2,
#         default=0,
#         null=True,
#         verbose_name="Durata",
#     )

    class Admin:
        list_filter = ("status", "assigned_to",)
        search_fields = ("^name", "modification_date", )
        date_hierarchy = "date_create"
        list_display = ("assigned_to", "name", "date_last_modify", "duration", )
        list_display_links = ("name", )
        list_per_page = 30
        ordering = ("name","-date_last_modify",)
        fields = (
            (None, {"fields": (
                ("status",),
                ("assigned_to",),
                ("name",),
                ("description",),
                ("date_last_modify", "duration",),
                ("comment",),
            ),}),
        )

    class Meta:
        verbose_name = "Action"
        verbose_name_plural = "Azioni"

    def __unicode__(self):
        return unicode(self.name)    

class Category(models.Model):
    code = models.CharField(max_length=5, primary_key=True)
    description = models.CharField(max_length=25)

    def __unicode__(self):
        return self.description
    
    class Admin: 
        list_display = ('code', 'description')
        search_fields = ('code', 'description')

    class Meta:
        verbose_name = _('Categories')
    
dispatcher.connect(evaluate_and_save,   signal=signals.pre_save,  sender=Ticket)
dispatcher.connect(mail_on_change,      signal=signals.post_save, sender=Ticket)

def add_group():
    from jungle.functions.tools import add_group_with_app_permissions
    
    add_group_with_app_permissions('Ticket', Ticket._meta.app_label)

dispatcher.connect(add_group, signal=signals.post_syncdb, weak=False)
