from jungle.timereport.models import *
from datetime import *

class IncompleteData(Exception): pass

class Total(object):
    """calculate the total effectively worked hours, overtime, missing
    based on transits. Take into consideration timetable profile and rounding
    rules. 

    It's supposed to be overwritten at least in the round method.

    Acts on transit elements (records)

    """
    columns = [r.column for r in Reason.objects.all()]
    def __init__(self, day=None, n=None):
        
        if day and day.is_holiday():
            self.force_work_is_extra = True
        else:
            self.force_work_is_extra = False
        self.day = day
        self.transits = {}  # key is the reason, value is a list of transits
        self._extra = 0
        self._missing = 0
        
        self.total = {}
        for column in self.columns:
            self.total[column] = timedelta()
        
    def add(self, transit):

        if not self.transits:
            self.profile = transit.profile
            self.isoweekday = transit.date_reference.isoweekday()
            
        if not transit.reason.code in self.transits:
            self.transits[transit.reason.code] = [transit]
        else:
            self.transits[transit.reason.code] += [transit]

    def compute(self):
        # each reason is treated differently since I want it to be valid also for
        # nested periods. eg: a permit inside a working day:
        # work: 8.00 -> 15.00 permit 11.00 -> 12.00
        # would be translated to 8 -> 11 (work) 11 -> 12 (permit) 12 -> 15 (work)
        # since the permit is programmed in advance it could overlap with
        # real transit taken from badg reader as in:
        # would be translated to 8 -> 11.05 (work) 11 -> 12 (permit) 12 -> 15 (work)

        #import pdb; pdb.set_trace()

        for reason_id, transits in self.transits.iteritems():
            reason = transits[0].reason
            if len(transits) %2:
                msg = "data for day %s seem to be incorrect" 
                raise IncompleteData(msg % transits[0].date_reference)

            delta = timedelta()
            transits.sort()

            for i in range(len(transits)-1, 0, -2):  # eg.:  [5,3,1]
                delta += self.round_chunk(
                    transits[i].date_time, transits[i-1].date_time, i)
                
#            self.total[reason.worked] += delta
            self.total[reason.column] += delta
            
    def get_diff(self):
        """
        
        """
        try:
            official = self.profile.get_total_working_seconds(self.isoweekday)
        except AttributeError:
            return

        
        diff = (self.total['worked'] + self.total['holiday']).seconds - official
        if diff > 0:
            self._extra = diff
        else:
            self._missing = -diff

        if self.force_work_is_extra:
            self._extra = self.total['worked'].seconds
            self._missing = 0

        if 'change' in self.total:
            self._extra -= self.total['change'].seconds

    @staticmethod
    def delta2str(delta):
        """in case delta is negative it prins a minus '-' """
        
        sign = ''
        if delta.days < 0:
            delta = abs(delta)
            sign = '-'
        h, rest = divmod(delta.seconds, 3600)
        m, sec = divmod(rest, 60)
        if h or m:
            return sign + time(h, m, sec).strftime("%H:%M")
        else:
            return ''


    def sec2str(self, sec):
        h, rest = divmod(sec, 3600)
        m, sec = divmod(rest, 60)
        return time(h, m, sec).strftime("%H:%M")

    def get_worked(self):
        return self.delta2str(self.total['worked'] )

    def get_holiday(self):
        return self.delta2str(self.total['holiday'] )

    def get_illness(self):
        return self.delta2str(self.total['illness'] )

    def get_extra(self):
        if self._extra:
            if self._extra < 0:
                return "-%s" % self.sec2str(self._extra * -1)
            return self.sec2str(self._extra )
        else:
            return ''

    def get_missing(self):
        if self._missing:
            return self.sec2str(self._missing )
        else:
            return ''

    def get_spare(self):
        return ''


    worked = property(get_worked)
    extra = property(get_extra)
    missing = property(get_missing)
    holiday = property(get_holiday)
    illness = property(get_illness)
    spare = property(get_spare)

    def round_chunk(self, datetime2, datetime1, odv):
        """this must be overloaded to implement different behaviour for single
        periods of the day"""
        
        return datetime2 - datetime1
    
    def round(self, datetime):
        """this must be overwritten so that it take into account local use/preference"""

        return datetime

    def get_totals(self):
        self.compute()

        self.total['worked'] = self.round(self.total['worked'])
        self.get_diff()
        self.get_worked()
        return (self.total['worked'], timedelta(seconds=self._extra),
                self.total['holiday'],self.total['illness'])

    def __str__(self):
        return self.compute()

    def __unicode__(self):
        return self.compute()
        
class PeriodTotal(object):
    columns = [r.column for r in Reason.objects.all()]
    def __init__(self, user_id):
        self.incomplete = False
        self.user_id = user_id
        self.total = {}
        for column in self.columns + ['extra']:
            self.total[column] = timedelta()

    def add(self, total_instance):
        try:
            w, e, h, i = total_instance.get_totals()
        except IncompleteData:
            #self.incomplete = True
            return

        self.total['worked']  += w
        self.total['extra']   += e
        self.total['holiday'] += h
        self.total['illness'] += i

    def get_worked(self):
        #import pdb; pdb.set_trace()
        if self.incomplete:
            return ''
        return delta2str(self.total['worked'])
        
    def get_extra(self):
        if self.incomplete:
            return ''
        return delta2str(self.total['extra'])
        
    def get_holiday(self):
        if self.incomplete:
            return ''
        return delta2str(self.total['holiday'])
        
    def get_illness(self):
        if self.incomplete:
            return ''
        return delta2str(self.total['illness'])

    worked = property(get_worked)
    extra  = property(get_extra)
    holiday = property(get_holiday)
    illness = property(get_illness)


def delta2str(delta):
    days = delta.days
    hours, spare = divmod(delta.seconds, 3600)
    spare = spare/750*15
    if spare == 60:
        hours += 1
        spare = '00'
    return "%s:%s" % (hours + 24*days, spare)


def round_delta_to_halfe(delta, border=15):
    """round the deltatime to closest .30 minutes
    return a tuple of deltas: the computed delta and the roundoff
    >>> round_delta_to_halfe(timedelta(hours=4, minutes=46))
    5:00:00
    >>> round_delta_to_halfe(timedelta(hours=4, minutes=44))
    4:30:00
    >>> round_delta_to_halfe(timedelta(hours=4, minutes=14))
    4:00:00
    >>> round_delta_to_halfe(timedelta(hours=4, minutes=16))
    4:30:00
    """    
    days = delta.days
    hours, spare_seconds = divmod(delta.seconds, 3600)
    halfes, seconds = divmod(spare_seconds, 1800)
    if seconds > border*60:
        roundoff = 1800-seconds
    else:
        roundoff = -seconds
        
    return (delta + timedelta(seconds=roundoff), timedelta(roundoff))

def round_datetime_to_halfe(date_time, border=15):
    date, time = date_time.date(), date_time.time()
    delta = timedelta(hours=time.hour, minutes=time.minute)
    
    hours, spare_seconds = divmod(delta.seconds, 3600)
    halfes, seconds = divmod(spare_seconds, 1800)
    if seconds > border*60:
        roundoff = 1800-seconds
    else:
        roundoff = -seconds
        
    return (delta + timedelta(seconds=roundoff), timedelta(roundoff))

    
