"""
This module provides a set of useful tools.

'dyn_import' is a function that imports in a transparent way the right
module and returns the class requested by a string.

Example:

from jungle.functions import tools

ModelObject = tools.dyn_import('model')

ModelResult = ModelObject.objects.all()



"""

from django.conf import settings

def add_group_with_app_permissions(group_name, app_name):
    """ This function is useful to create and return a Group having all
    permissions relatives to an application when a post_syncdb is called  """

    from django.contrib.contenttypes.models import ContentType
    from django.contrib.auth.models import Permission, Group

    cts = ContentType.objects.filter(app_label__exact = app_name)

    pms = Permission.objects.filter(content_type__in = cts)

    try: # group has already been created
        Group.objects.get(name = group_name)
    except:
        g = Group()
        g.name = group_name
        g.save()
        g.permissions = pms
        g.save()

def send_mail_template(template, context, subject, from_email, recipient_list):
    """ template is a valid mako .mako file;
       context is the dictionary sent to the template;
       recipient_list is of course a [] of email addresses. """
    
    from django.core.mail import send_mail
    from django.template import Context
    from jungle.template import mako_loader as mloader

    ENC = settings.DEFAULT_CHARSET
    t = mloader.get_template(template)
    c = Context(context)
    message = t.render(c).encode(ENC)
    send_mail(subject, message, from_email, recipient_list)

# def dyn_import(name, module_name=None, attr=None):
#     """
#     This function dinamically imports a Django Model given as string
#     using MODULES_MAP dictionary created by make_modules_map.py script.
#     Pay attention: model name in MODULES_MAP is a lowercase string.

#     you can get also get a module placed in the same dir using parameter
#     'module_name' e.g.::

#       FSOpts = dyn_import('ticket', module_name='fast_search')

#     will import module fast_search from the same directory as ticket.models.

#     """

#     name = name.lower()
#     elements = settings.MODULES_MAP[name]['import'].split('.')
#     Name = elements.pop()
#     object_name = attr or Name
#     if module_name:
#         elements.pop()  # get rid of 'models'
#         elements.append(module_name)
#     import_name = '.'.join(elements)

#     try:
#         module = __import__(import_name, {}, {}, [''])
#     except ImportError, e:
#         raise e
#     # object_name may be CamelCase
#     return getattr(module, object_name)

def import_custom(name, app_label, default=None):
    
    """try to import a function, class or module from:
       1. project.custom.app_label
       2. app_label module (where models lives)

    As an example you can import a personalization for ticket from
    jungle.ticket.fast_search or from my_project.custom.ticket.fast_search
    """

    prj = settings.SETTINGS_MODULE.replace('.settings','')
    try:
        import_str = "%s.custom.%s.%s" % (prj, app_label, name)
        return __import__(import_str, {}, {}, [''])
    except ImportError, e:
        pass

    try:
        from django.db.models.loading import get_app

        app = get_app(app_label).__name__.replace('.models', '')

        import_str = "%s.%s" % (app, name)
        return __import__(import_str, {}, {}, [''])
    except ImportError, e:
        pass


    return default
            

class Item(object):
    """ a simple way to turn an instance into a subscriptable obj
    return an object encoded into settings.DEFAULT_CARSET
    """
    
    def __init__(self, obj, enc=settings.DEFAULT_CHARSET):
        self.obj = obj
        self.ENC = enc
        
    def __getitem__(self, name):
        try:
            return getattr(self.obj, name).encode(self.ENC)
        except:
            ## integers and other do not have 'encode' attribute
            return getattr(self.obj, name)

def app_contrib_from_settings():
    from django.db import models

    f = {
        'uno' : models.CharField(maxlength=2),
        'due' : models.IntegerField()
        }
    return f

def app_in_settings(app):

    if app in settings.INSTALLED_APPS:
        return True

    return False
    
if __name__ == '__main__':
    import unittest
    import os
    
    os.environ['DJANGO_SETTINGS_MODULE'] = 'jobs.settings'
    
    class Test(unittest.TestCase):
        def test_import(self):
            from jungle.ticket.models import Ticket
            self.assertEqual(dyn_import('ticket'), Ticket)
            
    unittest.main()
