"""
The aim of this middleware is to allow people to have several different sites
that live in the same apache conf w/o using virtual domains (that is not always
possible/comfortable). If you want to reach you site as
http://localhost/cinemino, the way I use it is 

in settings.py:

    LOCATION  = '/cinemino/'
    # LOCATION = '/'  # this is for virtual host or develop server
    MEDIA_URL = LOCATION + 'static/
    ADMIN_MEDIA_PREFIX = LOCATION + 'media/'


and in apache2 conf file:

    Alias /cinemino/static/  /your/path/cinemino/static/
    <Location "/cinemino">
        SetHandler python-program
        PythonHandler django.core.handlers.modpython
        SetEnv DJANGO_SETTINGS_MODULE cinemino.settings
        PythonInterpreter cinemino
    </Location>

If you want to change the <Location> or if you want to move it directly to 
a virtual host it is sufficient to change settings.LOCATION and apache conf

in templates:

    any absolute link as in href="/abc" or next=/account will be translated to
    adding a triling LOCATION 

See below to know how urls.py must be changed.

NOTES:
 * if you use generic views that use post_save_redirect be sure to pass an
   already correct URL as it will not pass throught this middleware!

 * css are not served anyhow. Use Location in apache for this case.

 * you can use // as in href="//..." to force adding a location in ambiguous
   cases. In the future may become the only modification.
"""


from django.conf import settings
from jungle.functions.conf import *

import re

class AddLocation:
    """add settings.LOCATION to any relative path.
    If the path is relative (no leading '/') and settings.LOCATION is defined
    we substitute that. 
    """
    def __init__(self, location=None):
        if location is not None:
            ## just for unittest purpose
            self.replace = r'\1\2%s/'  % location.rstrip('/') 
            self.LOCATION = location
            return
        
        try:
            self.LOCATION = settings.LOCATION
        except AttributeError, e:
            self.LOCATION = ''

        self.apache = is_running_in_apache()
        self.replace = r'\1\2%s/'  % settings.LOCATION.rstrip('/') 
        
    def process_response(self, request, response):
        if self.LOCATION and \
               response['Content-Type'].startswith('text/html'):
            response.content = self.add_location(response.content, request.path)
        return response

    def add_location(self, content, path=''):
        # No 'add_location' for admin
        if path and path.startswith('%s/admin' % self.LOCATION.rstrip('/')):
            return content

        c = re.compile(r'''
           (href|src|action|next)
           (=[\'"]?)//?                                                
        ''', re.VERBOSE)

        content = c.sub(self.replace, content)
        return content

    def add_location_old(self, content, path=''):
        # No 'add_location' for admin
        if path and path.startswith('%s/admin' % self.LOCATION.rstrip('/')):
            return content

        c = re.compile(r'''
           (href|src|action|next)
           (=[\'"]?(?!              # negative lookahead: =" matches if not followed by...
                     [/#]{1,1}|         # absolute path & named anchor
                     [a-z]+://|    # [a-z]+:// a protocol as href="http://domain"
                                   # should be left untouched
                     [-&@~%a-zA-Z0-9_.#]*  # any link w/o "/"
                     /?[\'"]))       # empty ones included
                                   
        ''', re.VERBOSE)

        content = c.sub(self.replace, content)
        c = re.compile(r'''
           (next=|url\([\'"]?    )  # does not have a "
           ((?! /{1,1}|                 # absolute url
                [a-z]+://  ))      # [a-z]+:// a protocol, href="http://domain"
                                   # should be left untouched
        ''', re.VERBOSE)

        content = c.sub(self.replace, content)
        c = re.compile(r'(href=|src=|action=)(")//')
        content = c.sub(self.replace, content)
        
        return content

def lpatterns(prefix, *tuples):
    """modify patterns to keep settings.LOCATION into consideration
    You are supposed to import lpatterns *beyond* patterns in the main urls.py

       from django.conf.urls.defaults import include
       from jungle.middleware.add_location import lpatterns
    
    """
    from django.conf.urls.defaults import patterns
    import re
    
    try:
        LOCATION = settings.LOCATION
    except AttributeError, e:
        LOCATION = ''

    if LOCATION in ('','/'):
        replace = '^'
    else:
        replace = '^%s/' % LOCATION.strip('/')
        
    pattern_list = []
    for t in tuples:

        if len(t) == 2:
            t2 = (re.sub(r'^\^', replace, t[0]), t[1])
        else:
            t2 = (re.sub(r'^\^', replace, t[0]), t[1], t[2])
        pattern_list.append( t2)
    return patterns(prefix, *pattern_list)


if __name__ == '__main__':
    import unittest

    class Test(unittest.TestCase):
        def setUp(self):
            self.a = AddLocation(location='/')
            self.b = AddLocation(location='/cinemino/')
            self.a.apache = False
            self.b.apache = False
            
        def test_a_Empty(self):
            txt = '<a href="">uno</a>'
            self.assertEqual(txt, self.a.add_location(txt))
        
        def test_b_Empty(self):
            txt = '<a href="">uno</a>'
            self.assertEqual(txt, self.b.add_location(txt))
        
        def test_a_File(self):
            txt = '<a href="project.png">uno</a>'
            self.assertEqual(txt, self.a.add_location(txt))

        def test_b_File(self):
            txt = '<a href="project.png">uno</a>'
            self.assertEqual(txt, self.b.add_location(txt))

        def test_a_RelativeDir(self):
            txt = '<a href="project.png">uno</a>'
            self.assertEqual('<a href="project.png">uno</a>',
                             self.a.add_location(txt))

        def test_a_URL(self):
            txt = '<a href="http://domain:543/ok.png">uno</a>'
            self.assertEqual(txt, self.a.add_location(txt))

        def test_b_URL(self):
            txt = '<a href="http://domain:543/ok.png">uno</a>'
            self.assertEqual(txt, self.b.add_location(txt))

        def test_b_RelativeDir(self):
            txt = '<a href="project.png">uno</a>'
            self.assertEqual('<a href="project.png">uno</a>',
                             self.b.add_location(txt))
        def test_a_Next(self):
            txt  = '<a href="//login/?next=//film/vota">uno</a>'
            txt2 = '<a href="/login/?next=/film/vota">uno</a>'
            self.assertEqual(txt2, self.a.add_location(txt))
            
        def test_b_Next(self):
            txt  = '<a href="/login/?next=/film/vota">uno</a>'
            txt2 = '<a href="/cinemino/login/?next=/cinemino/film/vota">uno</a>'
            self.assertEqual(txt2, self.b.add_location(txt))
            
        def test_a_Next2(self):
            txt  = '<a href="login/?next=//film/vota">uno</a>'
            txt2 = '<a href="login/?next=/film/vota">uno</a>'
            self.assertEqual(txt2, self.a.add_location(txt))
            
        def test_b_Next2(self):
            txt  = '<a href="login/?next=//film/vota">uno</a>'
            txt2 = '<a href="login/?next=/cinemino/film/vota">uno</a>'
            self.assertEqual(txt2, self.b.add_location(txt))
            
#         def test_a_URLcss(self):
#             txt  = 'url(static/image/film.gif)'
#             txt2 = 'url(/static/image/film.gif)'
#             self.assertEqual(txt2, self.a.add_location(txt))
            
#         def test_b_URLcss(self):
#             txt = 'url(static/css/testo.gif)'
#             txt2 = 'url(/cinemino/static/css/testo.gif)'
#             self.assertEqual(txt2, self.b.add_location(txt))
            
#         def test_a_URLcss2(self):
#             txt  = 'url(static/image/film.gif)'
#             txt2 = 'url(/static/image/film.gif)'
#             self.assertEqual(txt2, self.a.add_location(txt))
            
#         def test_b_URLcss2(self):
#             txt = "url('static/css/testo.gif')"
#             txt2 = "url('/cinemino/static/css/testo.gif')"
#             self.assertEqual(txt2, self.b.add_location(txt))
            
#         def test_a_URLcss3(self):
#             txt  = 'url("static/image/film.gif")'
#             txt2 = 'url("/static/image/film.gif")'
#             self.assertEqual(txt2, self.a.add_location(txt))
            
#         def test_b_URLcss3(self):
#             txt = 'url("static/css/testo.gif")'
#             txt2 = 'url("/cinemino/static/css/testo.gif")'
#             self.assertEqual(txt2, self.b.add_location(txt))
            
        def test_a_RelFile(self):
            txt  = '<LI><A href="users_guide.html#SECTION0002">1 Intro</a>'
            self.assertEqual(txt, self.b.add_location(txt))
            
        def test_b_RelFile(self):
            txt  = '<LI><A href="users_guide.html#SECTION0002">1 Intro</a>'
            self.assertEqual(txt, self.b.add_location(txt))
            
        def test_a_AdmLink(self):
            txt  = '<a href="password_change/">'
            self.assertEqual(txt, self.b.add_location(txt))
            
        def test_b_AdmLink(self): 
            txt  = '<a href="password_change/">'
            self.assertEqual(txt, self.b.add_location(txt))

        def test_a_Simple(self):
            txt  = '<a href="//documentation">documentation</a></h2>'
            txt2  = '<a href="/documentation">documentation</a></h2>'
            self.assertEqual(txt2, self.a.add_location(txt))
            
        def test_b_Simple(self): 
            txt  = '<a href="//documentation">documentation</a></h2>'
            txt2  = '<a href="/cinemino/documentation">documentation</a></h2>'
            self.assertEqual(txt2, self.b.add_location(txt))

        def test_a_AdminPath(self):
            txt = '<th scope="row"><a href="auth/group/">Gruppi</a></th>'
            self.assertEqual(txt, self.a.add_location(txt, path='/admin'))

        def test_b_AdminPath(self):
            txt = '<th scope="row"><a href="auth/group/">Gruppi</a></th>'
            self.assertEqual(txt, self.b.add_location(txt, path='/cinemino/admin'))

            

    unittest.main()
