from django.conf import settings
from django.http import HttpResponse
from django.template import TemplateDoesNotExist
from django.template.context import Context, RequestContext
from mako.lookup import TemplateLookup
from mako.template import Template
from mako.exceptions import TopLevelLookupException
import os, re
from common import app_dirs

'''
configurations:
    MAKO_TEMPLATE_DIRS:
        A tuple, specify the directories in which to find the mako templates, 
        just like TEMPLATE_DIRS .
        default value is ('mako_templates',)
    MAKO_MODULE_DIR:
        A string, if specified, all of the compiled template module files will be
        stored in this directory.
    MAKO_MODULENAME_CALLABLE:
        A callable, if MAKO_MODULE_DIR is not specified, this will be
        used to determine the filename of compiled template module file.
        See [http://www.makotemplates.org/trac/ticket/14]
        Default to the function `default_module_name`, which
        just appends '.py' to the template filename.
    MAKO_INPUT_ENCODING = "utf-8"
    MAKO_PREPROCESSOR = [replace]
    MAKO_IMPORTS = 
    
'''

def replace(text):
    """standard prerocessor to implement $!, $!?, $?!, $?"""
    text = re.sub('\$(?P<mode>[!?]+)(?P<name>\w+)[:|]?(?P<def>[${}.\w]*)', sub, text)
    return text

def sub(m):
    """substitution called in replace to implement $!, $?, $!?, $?!
    """
    mode = m.group('mode')
    if mode == '!':
        func = 'ifdef'
    elif mode == '?':
        func = 'ifdef_in_request'
    elif mode == '!?':
        func = 'ifdef_local_request'
    elif mode == '?!':
        func = 'ifdef_request_local'
        
    if m.group('def').startswith('$'):
        default = re.sub('[${}]','', m.group('def'))
        return "${self.%s('%s',%s)}" % (func, m.group('name'), default)
    return "${self.%s('%s','%s')}" % (func, m.group('name'), m.group('def'))

MAKO_TEMPLATE_DIRS = getattr(settings, 'MAKO_TEMPLATE_DIRS', settings.TEMPLATE_DIRS)
MAKO_MODULE_DIR = getattr(settings, 'MAKO_MODULE_DIR',
                          settings.CWD + '/mako_modules')
MAKO_FILTERS = getattr(settings, 'MAKO_FILTERS', ['jun.smart_uni'])
MAKO_INPUT_ENCODING = getattr(settings, 'MAKO_INPUT_ENCODING', 'utf-8')
MAKO_IMPORTS = getattr(settings, 'MAKO_IMPORTS',
                       ['from jungle.templatetags import menu, dj, jun, forms'])
MAKO_PREPROCESSOR = getattr(settings, 'MAKO_PREPROCESSOR', [replace])


app_template_dirs = []
for app_dir in app_dirs:
    template_dir = os.path.join(app_dir, 'templates')
    if os.path.isdir(template_dir):
        app_template_dirs.append(template_dir)

template_dirs = MAKO_TEMPLATE_DIRS
template_dirs += tuple(app_template_dirs)

def default_module_name(filename, uri):
    '''
    Will store module files in the same directory as the corresponding template files.
    detail about module_name_callable, go to 
    http://www.makotemplates.org/trac/ticket/14
    '''
    return filename+'.py'

module_dir = MAKO_MODULE_DIR
if module_dir:
    lookup = TemplateLookup(directories=template_dirs,
                            module_directory=module_dir,
                            input_encoding=MAKO_INPUT_ENCODING,
                            imports=MAKO_IMPORTS,
                            default_filters=MAKO_FILTERS,
                            preprocessor=MAKO_PREPROCESSOR,
                            )
else:
    module_name_callable = getattr(settings, 'MAKO_MODULENAME_CALLABLE', None)
    if callable(module_name_callable):
        lookup = TemplateLookup(directories=template_dirs,
                                modulename_callable=module_name_callable,
                                )
    else:
        lookup = TemplateLookup(directories=template_dirs,
                modulename_callable=default_module_name)
def select_template(template_name_list):
    for template_name in template_name_list:
        try:
            return lookup.get_template(template_name)
        except TopLevelLookupException:
            pass

    ### worse then some lines below, since here I just look for one
    ### template_name intead of the list I was passed
    from django.template.loader import find_template_source
    find_template_source(name=template_name)

    raise TemplateDoesNotExist, 'mako templates: '+', '.join(template_name_list)

def get_template(template_name):
    if isinstance(template_name, (list, tuple)):
        t = select_template(template_name)
    else:
        try: 
            t = lookup.get_template(template_name)
        except TopLevelLookupException, e:
            pass
    try:
        t.render = render.__get__(t, Template)
        return t
    except:
        ## a bit hackish. find_template_loader would normally be
        ## invoked by get_template (in django template system) and populates
        ## template_source_loaders that is used by technical_500_response
        ## to render a nice error page for missing templates
        from django.template.loader import find_template_source
        find_template_source(name=template_name)

        raise TemplateDoesNotExist, 'mako templates: '+template_name_list

def render_to_response(template_name, dictionary=None, context_instance=None):
    if isinstance(template_name, (list, tuple)):
        template = select_template(template_name)
    else:
        template = get_template(template_name)

    dictionary = dictionary or {}
    if context_instance is None:
        context_instance = Context(dictionary)
    else:
        context_instance.update(dictionary)
    data = {}
    [data.update(d) for d in context_instance]

    return HttpResponse(template.render_unicode(**data))

def render_response(req, *args, **kwargs):
    kwargs['context_instance'] = RequestContext(req)
    return render_to_response(*args, **kwargs)

def render(self, context):
    """used in get_template  to add a method to a Template instance::
    
       t.render = render.__get__(t, Template)

    """
    
    C = {}
    for i in context:
        C.update(i)
    try:
        ret = self.render_unicode(**C)
    except NameError, e:
        if settings.DEBUG:
            #print vars(self)
            #print "Keys in context:", C.keys()
            pass
        raise e
    
    return ret

def get_def_from_template(template_name, def_name='layout', data={}):
    t = get_template(template_name)
    try:
        #print t.get_def(def_name).render_unicode(**data)
        return t.get_def(def_name).render_unicode(**data)
    except NameError, e:
        if settings.DEBUG:
            pass
            #print "Keys in context:", data.keys()
        raise e
