from django.core.xheaders import populate_xheaders
from jungle.template import mako_loader as mloader
from django import newforms
from django.newforms.models import form_for_model, form_for_instance
from django.db.models import FileField, BooleanField
from django.contrib.auth.views import redirect_to_login
from django.template import RequestContext
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.core.exceptions import ObjectDoesNotExist, ImproperlyConfigured
from django.utils.translation import gettext
from jungle.db.formfield import ffc
#from django.db import IntegrityError
import inspect

def create_object(request, model, form=None, template_name=None,
        template_loader=mloader, extra_context=None, post_save_redirect=None,
        login_required=False, follow=None, context_processors=None):
    """
    Generic object-creation function.

    Templates: ``<app_label>/<model_name>_form.html``
    Context:
        form
            the form wrapper for the object

    NOTE: differently from django source I allow form to be passed. form can be
          both a class o a (bound) instance
    """
    if extra_context is None: extra_context = {}
    if login_required and not request.user.is_authenticated():
        return redirect_to_login(request.path)

    if not form:
        Form = form_for_model(model, formfield_callback=ffc())
        form = Form
    else:
        if inspect.isclass(form):
            Form = form
        else:
            Form = form.__class__

    if request.POST:
        # If data was POSTed, we're trying to create a new object
        new_data = request.POST.copy()
        new_file = None
        
        if model._meta.has_field_type(FileField):
            new_file = request.FILES
            #new_data.update(request.FILES)

        # Check for errors
        if new_file:
            form = Form(new_data, new_file)
        else:
            form = Form(new_data)
        
        if form.is_valid():
            # No errors -- this means we can save the data! except for db problems
            new_object = form.save()
            #try:
            #    pass
            #except IntegrityError:
            #    pass
                #new_object = None
                #form.errors['db'] = "Integrity Error from the db"
                
            if True:   # new_object:
                if request.user.is_authenticated():
                    request.user.message_set.create(message=gettext("The %(verbose_name)s was created successfully.") % {"verbose_name": model._meta.verbose_name})

                # Redirect to the new object: first by trying post_save_redirect,
                # then by obj.get_absolute_url; fail if neither works.
                if post_save_redirect:
                    if hasattr(new_object, '__dict__'):
                        result = new_object.__dict__
                    else:
                        result = {}
                    return HttpResponseRedirect(post_save_redirect % result)
                elif hasattr(new_object, 'get_absolute_url'):
                    return HttpResponseRedirect(new_object.get_absolute_url())
                else:
                    raise ImproperlyConfigured("No URL to redirect to from generic create view.")
    else:
        ## if form is already an instance we use it
        if inspect.isclass(form):
            form = Form()

    # Create the FormWrapper, template, context, response
    #form = oldforms.FormWrapper(manipulator, new_data, errors)
    if not template_name:
        template_name = "%s/%s_form.html" % (model._meta.app_label, model._meta.object_name.lower())
    t = template_loader.get_template(template_name)
    c = RequestContext(request, {
        'form': form,
    }, context_processors)
    for key, value in extra_context.items():
        if callable(value):
            c[key] = value()
        else:
            c[key] = value
    return HttpResponse(t.render(c))

def update_object(request, model, form=None, object_id=None, slug=None,
        slug_field=None, template_name=None, template_loader=mloader,
        extra_context=None, post_save_redirect=None,
        login_required=False, follow=None, context_processors=None,
        template_object_name='object'):
    """
    Generic object-update function.

    Templates: ``<app_label>/<model_name>_form.html``
    Context:
        form
            the form wrapper for the object
        object
            the original object being edited
    """
    if extra_context is None: extra_context = {}
    if login_required and not request.user.is_authenticated():
        return redirect_to_login(request.path)

    # Look up the object to be edited
    lookup_kwargs = {}
    if object_id:
        lookup_kwargs['%s__exact' % model._meta.pk.name] = object_id
    elif slug and slug_field:
        lookup_kwargs['%s__exact' % slug_field] = slug
    else:
        raise AttributeError("Generic edit view must be called with either an object_id or a slug/slug_field")
    try:
        object = model.objects.get(**lookup_kwargs)
    except ObjectDoesNotExist:
        raise Http404, "No %s found for %s" % (model._meta.verbose_name, lookup_kwargs)

    if not form:
        Form = form_for_instance(object, formfield_callback=ffc())
    else:
        if inspect.isclass(form):
            Form = form
        else:
            Form = form.__class__

    from django.utils.datastructures import MultiValueDict
    data = MultiValueDict()
    boolean_fields = []
        
    for f in object._meta.fields:
        data.setlist(f.name, [f.value_from_object(object)])
        if isinstance(f, BooleanField):
            boolean_fields.append(f.name)
    
    for f in object._meta.many_to_many:
        data.setlist(f.attname,  [getattr(x, x._meta.pk.name) for x in f.value_from_object(object)])

    if request.POST:

        ## IMPORTANT: we don't want to overwrite values that are not from POST
        for i in request.POST:
            data.setlist(i, request.POST.getlist(i))

        for i in boolean_fields:
            if i not in request.POST:
                data.setlist(i, [False])

        if model._meta.has_field_type(FileField):
            new_file = request.FILES
        try:
            form = Form(data, new_file)
        except UnboundLocalError, e:
            form = Form(data)
        
        ## FIXME: errors
        #errors = manipulator.get_validation_errors(new_data)
        #manipulator.do_html2python(new_data)
        if form.is_valid():
            object = form.save()

            if request.user.is_authenticated():
                request.user.message_set.create(message=gettext("The %(verbose_name)s was updated successfully.") % {"verbose_name": model._meta.verbose_name})

            # Do a post-after-redirect so that reload works, etc.
            if post_save_redirect:
                return HttpResponseRedirect(post_save_redirect % object.__dict__)
            elif hasattr(object, 'get_absolute_url'):
                return HttpResponseRedirect(object.get_absolute_url())
            else:
                raise ImproperlyConfigured("No URL to redirect to from generic create view.")
    else:
        errors = {}

    if not form or inspect.isclass(form):
        form = Form(data)
    else:
        ## is it's not a class is an instance, just use it as is
        pass

    #form = Form(data)
    if not template_name:
        template_name = "%s/%s_form.html" % (model._meta.app_label, model._meta.object_name.lower())
    t = template_loader.get_template(template_name)
    c = RequestContext(request, {
        'form': form,
        template_object_name: object,
    }, context_processors)
    for key, value in extra_context.items():
        if callable(value):
            c[key] = value()
        else:
            c[key] = value
    response = HttpResponse(t.render(c))
    populate_xheaders(request, response, model, getattr(object, object._meta.pk.attname))
    return response

def delete_object(request, model, post_delete_redirect,
        object_id=None, slug=None, slug_field=None, template_name=None,
        template_loader=mloader, extra_context=None,
        login_required=False, context_processors=None, template_object_name='object'):
    """
    Generic object-delete function.

    The given template will be used to confirm deletetion if this view is
    fetched using GET; for safty, deletion will only be performed if this
    view is POSTed.

    Templates: ``<app_label>/<model_name>_confirm_delete.html``
    Context:
        object
            the original object being deleted
    """
    if extra_context is None: extra_context = {}
    if login_required and not request.user.is_authenticated():
        return redirect_to_login(request.path)

    # Look up the object to be edited
    lookup_kwargs = {}
    if object_id:
        lookup_kwargs['%s__exact' % model._meta.pk.name] = object_id
    elif slug and slug_field:
        lookup_kwargs['%s__exact' % slug_field] = slug
    else:
        raise AttributeError("Generic delete view must be called with either an object_id or a slug/slug_field")
    try:
        object = model._default_manager.get(**lookup_kwargs)
    except ObjectDoesNotExist:
        raise Http404, "No %s found for %s" % (model._meta.app_label, lookup_kwargs)

    if request.method == 'POST':
        object.delete()
        if request.user.is_authenticated():
            request.user.message_set.create(message=gettext("The %(verbose_name)s was deleted.") % {"verbose_name": model._meta.verbose_name})
        return HttpResponseRedirect(post_delete_redirect)
    else:
        if not template_name:
            template_name = "%s/%s_confirm_delete.html" % (model._meta.app_label, model._meta.object_name.lower())
        t = template_loader.get_template(template_name)
        c = RequestContext(request, {
            template_object_name: object,
        }, context_processors)
        for key, value in extra_context.items():
            if callable(value):
                c[key] = value()
            else:
                c[key] = value
        response = HttpResponse(t.render(c))
        populate_xheaders(request, response, model, getattr(object, object._meta.pk.attname))
        return response













